channels = ['@ukrultras', '@aeroplaneta', '@GadgetsForSale', '@MaksPodzigun', '@hmarochos', '@SerialList', '@kolhumor',
             '@dekanat_fmm', '@ukrtour', '@mudrost_mem', '@UkrMemesmineproblemes', '@infOdessa', '@ain_ua_tg',
             '@oltsyaka', '@lowcostua', '@KaterynaZoryana', '@znoonaa200', '@kinoman_ua', '@tourstanok_UA',
             '@rozetka_travel', '@klekscourses', '@shedevryreklamy', '@odessitua_com', '@TL_news', '@inlvivinua',
             '@navtg', '@osssumdu', '@aquotes', '@esportsua', '@jopnuta_podruga', '@IF112', '@itkpi', '@TravelTango',
             '@bikeua2eu', '@acmodasi_ua', '@YaKurkul', '@SBUkr', '@joinNBU', '@trubuprorvalo', '@shosneto',
             '@uacosmos', '@agrotax', '@traktoristua', '@atlasweekend', '@crazybeeua', '@vncomua', '@cultureznu',
             '@pidsluhano_ukraine_ua', '@ua_covid_19_ua', '@black_gymor', '@historydayua', '@ubereatsua', '@ziznanya',
             '@ukrlit', '@bedeveloped', '@novytv', '@dw_travel', '@hochu_v_kpi', '@illienko', '@spuf_knu', '@CustomsUA',
             '@dumkyprovazhlyve', '@glubinaz', '@eduhubinua', '@vinlife24', '@gastroprikhozhane', '@spravagromad',
             '@developmentbooks', '@happymonday', '@ticketsbox', '@MiComUA', '@Stankokulemet', '@travelsInfo',
             '@pr_perfect', '@nulp_prepod', '@HORECA_DNEPR_RABOTA', '@Ags_Donbass', '@med3pediatricians',
             '@ukrpravda_news', '@virsholub', '@Konstlaw', '@ikitb', '@paliturk', '@pmkagency', '@psyhologya_zhyttya',
             '@PM_Shmigal', '@chanel24', '@promandry', '@zno_ukr', '@myvolia', '@prometheusmooc', '@KanalSmihu',
             '@pani_vchytelka', '@sobakam', '@superagronomcom', '@mainacademy', '@samburska_giveaway', '@tusamyaso',
             '@box_of_pandora', '@anecdotes_ua', '@poltavanovosti', '@offer_blockchain', '@lab_of_childhood',
             '@taka_polsha', '@marusyazvirobiy', '@knteu_vstup', '@brodetsky_2', '@Triukhan', '@pavlovdrop',
             '@ukrzaliznica', '@telegrafdesign', '@GolosZmin', '@istukrmemes', '@vkadr', '@nmu_file_zanuch',
             '@Poltava_live', '@memasy_dlya_masy', '@ukr_poll', '@LvivMisto', '@cool_advice', '@GreenSommelier',
             '@zhuk_ua', '@gurt11', '@uasukr', '@dkkadrovikua', '@politicusvulgaris', '@americaua',
             '@Deutsche_Sprechweise', '@allonews', '@open_ukraine', '@christiansua', '@dcpgodmemes', '@kyivpatrol',
             '@Finance_ukr', '@VolunteerCountry', '@cargo1_auto_from_usa', '@scistudsoc', '@ukr_culinar',
             '@starapoltava', '@Football_Transfery', '@vzapertiua', '@dou_odesa', '@baptistyouthua',
             '@football_and_more', '@FashionUkraine', '@virshi_ukr', '@iod_media', '@Your_TURN', '@jerryheil',
             '@turizmua', '@kyivworkz', '@zhdanovadropshipping', '@usionline', '@AnimeKultura', '@koronavirus_news_ua',
             '@baldtrue', '@ukr_artists', '@orestzub', '@dmytro_dubilet', '@V_N_ZHEREBNIUK_DECEBAL', '@studlava',
             '@svstn', '@zakupki_prom_ua', '@vstuposvita', '@kpi_666', '@Moviestape', '@Chernigov_ua', '@Konkretyka',
             '@SnowDeal', '@profkomkarazina', '@prostirua', '@m_y_day', '@studwaydiem', '@IT_v_UA', '@nightflow',
             '@uceqa', '@opendataukraine', '@easyxpress', '@fyouall', '@thebig4', '@ukrainskinovosti', '@psagentclub',
             '@ear_on', '@g_untitled', '@authors_anonymous', '@sharaievskyimarketing', '@VolodymyrAriev', '@platinaKPI',
             '@ridna_ua', '@justbus', '@mdfashionua', '@ikrus', '@textyorgua', '@studway', '@afishaskidok',
             '@taburetkax', '@tourstanok_Lviv', '@zakononline2019', '@UkropInfo', '@linuvuyblog', '@sraka_motyka',
             '@mistotravel', '@pbikni', '@cxemu', '@novinach', '@yaizakon', '@zekharkiv2019', '@franchua',
             '@iamgoodatlviv', '@deshevihoteli', '@ukrainanovosti', '@dtp18plus', '@rabota_work_ua', '@fri_ua',
             '@megogolive_ua', '@dmytro_karpov', '@emailingskills', '@mistoMarii', '@poltavtsev01', '@hyperthelia',
             '@dragomanivtsi_FPP', '@lyapotagoods', '@goodnewsforyou', '@nlu_live', '@yourchurchill', '@FedorivHub',
             '@artius_law_firm', '@higherspro', '@PriceTimeBest', '@interesting_all', '@TCH_channel', '@kpi_trade',
             '@rooms_lviv', '@sportlife_ua_official', '@zapas_slv', '@StudentWanted', '@solovei_ihor',
             '@englishdictionaryua', '@posad_kh', '@medicinainfo', '@kpi_look', '@beinprogress', '@lifecell_campus',
             '@uaAutoNews', '@AlonaShkrum', '@dmelomaniya', '@politmedia22', '@jccnaukma', '@esnkyiv', '@brokers_diary',
             '@remote_work_ua', '@footboomcom', '@znoua1', '@watsonsukraine', '@ukrainian_map', '@operativeukr',
             '@mvs_ukraine', '@zhatkz', '@kermoua', '@your_chance_ua', '@memolanka', '@vybir_profesii', '@xchernivtsi',
             '@ukrzoloto', '@upc_news', '@dekanat_fel', '@dekanatfsp', '@suspilnekherson', '@ragoolee', '@uvidpustku',
             '@smachni_retsepty', '@cyberknu', '@pimpENgUA', '@marliua', '@ukr_lawyer', '@pskpi', '@alfabankua',
             '@docsm', '@chytalka', '@impulse_ukr', '@rivne1tv', '@bezocinok', '@InfoResist', '@KarandashUA',
             '@jb_girls', '@films_radnik', '@rozumschoolgeo', '@ZeRada1', '@thebabel', '@agrorobota_agrokholdynhy',
             '@strichka_ua', '@uabrand', '@magellanuz', '@minimalka', '@karpliukV', '@Shaleniy_Kit', '@NIZHN0',
             '@lnuvmb', '@uafootballcom', '@fat_zero', '@atyznaw', '@stepbystepfluency', '@Instahusband',
             '@legalhub_online', '@taras_shevchenko', '@ukr_opros', '@zvyk_ua', '@ua_danylo', '@mikrozaim_ua',
             '@odessaopt7km', '@TimeToTravel', '@drive_pdr_znay', '@gomolod', '@watchingua', '@NPUgurt',
             '@hromadske_ua', '@postironichniymetamodern', '@artehvact', '@superskidki_ua', '@podslyshenopolice',
             '@agrorobota_kyiv', '@mozofficial', '@pogodaUnian', '@opentv_media', '@map_stat_dat', '@istzno',
             '@placeforgirls', '@Ukr_lit_ZNOcloud', '@anime_pro', '@BusinessOpportunitiesKyiv', '@kyiv_with_kids',
             '@kvashnevskyiFM', '@SaleClubUkraine', '@mouselab', '@OnlineUkraine', '@histph', '@vantazhivka', '@ITCUA',
             '@nach_shtabu', '@ternopilwork', '@tv7mrpl', '@ukr_armed_forces', '@OstanniyCapitalist', '@RBC_ua_news',
             '@lanetua', '@automarket_ua', '@EnglishPremierLeagu', '@kkchannel79', '@darkside_ami', '@chernigovnovosti',
             '@belka_space', '@Koronavirus_info_moz', '@t_oday', '@houseofeurope', '@lingvaskills', '@bdisKNU',
             '@allNMU', '@bioblogic', '@philiszanka', '@teleobuh', '@severodonetsk', '@underrrcoverrr', '@econom_notes',
             '@bluzka_ua', '@lubeavtokhm', '@polkazov', '@uapentecostal', '@viktoRRocha', '@novrek', '@AgroPolitCom',
             '@malevych', '@Ukrainian_language', '@trclubava', '@agrorobota_trading', '@luxoptica_ua', '@poltava_to',
             '@FAKTY_ICTV', '@avramenko_chanel', '@Dreyph', '@ukrapmemes', '@orden83', '@carpaty', '@kyivosvita',
             '@backpackersua', '@lviv_it', '@Bandershtatfest', '@thelnu', '@space_job', '@WikiUa', '@aviasales_ua',
             '@brother_junior', '@blyadskiy_kharkov', '@cheaptripnow', '@AliExpress_do10', '@popkornua', '@tuiua',
             '@wtfbit', '@kino_divan', '@hardsmoke', '@snuwixgovno', '@ze_english', '@Ukranian_News', '@elucidation_gw',
             '@ukr_mova', '@theGAMEua', '@SportsHealthUa', '@nubip1898', '@AMB_islnd', '@oficialvmr', '@NikArtshoes',
             '@fainaFrancia', '@medic_ua', '@vnlviv', '@prykoly_ua', '@Vinnitsia_ua', '@kyivconcert', '@ua112',
             '@slavinska_reads', '@UkrFin', '@zaichenkoteam', '@BeerMarketUkraine', '@OnePlusOneChannel', '@pingvinpro',
             '@MilingoJournalUA', '@discover_ukraine', '@PDRUA', '@telegra4chukr', '@TheMensHealth', '@NASS_of_ART',
             '@zaxidfest', '@novostyukraine', '@drinkery_ukraine', '@lessmess', '@ukr2eng', '@ilonkis_z', '@offer_ua',
             '@utlgr', '@dniprofilm', '@smokesomile', '@Kyiv_Live', '@Lv1256', '@themanagerblog', '@karazinuniver1804',
             '@technologiiua', '@Afisha_odessaua', '@ZNlife', '@vso_ukr', '@JYSK_ua', '@chemood_tour',
             '@MOZ_Ukraine_official', '@the_old_chest', '@censor_net', '@osnovamaibutnoho', '@hotlinefinance',
             '@zedigital', '@geographia_znocloud', '@nauhub', '@presinfokpi', '@ngo_insight', '@pictureshistory',
             '@SharkCo', '@kpi_work', '@iplanua', '@odessaligasmeha', '@forbes_UAA', '@olkom_home',
             '@orjeunesseofficial', '@PokemonGOnewsUKR', '@if1711', '@sriblotroyandy', '@bykvu', '@DynamoK',
             '@top1top10', '@conservatorjournal', '@HoReCa_Premium', '@galkaifuanews', '@Ukr_mova_ZNOcloud',
             '@mikhaildobkin', '@progressevi', '@ukrmovafi', '@ua_kataloge_oriflame', '@MyLifeinukraine', '@novyny',
             '@lavinamallua', '@nationalcorps', '@kgbfiles', '@ua_hottours', '@profkom_nau', '@cybervacancies',
             '@visnyk_ua', '@krasa_Vn', '@studtravel', '@businessweekua', '@nighthistory', '@oneplusonevideo',
             '@off_the_record', '@Sho_Tama', '@KyivOperativ', '@ZeFuckTy', '@ty_vdoma', '@uamfa', '@korisno',
             '@eventskyiv', '@novoe_vremya', '@slovo_dnya_ukr', '@vmiske', '@elitnews', '@voxcheck', '@kyiv_meet',
             '@smartan', '@uacatalog', '@Leadershipforyou', '@dailymetal', '@kpitv', '@audiounn', '@chomu_tse_tak',
             '@insta_icon', '@dnepr_ladies', '@metrainee', '@promocode_ua', '@iventKHM', '@cowoguru', '@english_ukr',
             '@eventlviv', '@sunvery_recruiting', '@radio_kottans', '@uacomics', '@chernivtsigram', '@ua_vd_i_vn',
             '@uanother', '@sr_kpi', '@tasty_flashbacks', '@kinchik_pid_vinchik', '@golovaOP', '@agrosem_ua',
             '@starlev', '@nbdwn', '@ukrlit_mems', '@Metro6', '@donetsk_kurs', '@KIVAERO', '@pnulife', '@league_uba',
             '@vidpoviday', '@kpi_777', '@znohistoryua200', '@VR_UA', '@stas_memes', '@naplanetikino', '@ukrarticles',
             '@ubaua', '@odeparh', '@revdiachannel', '@DesJobUa', '@skyfreechannel', '@cheap_europe', '@lvivjob',
             '@hovoryt_Ukraina', '@timetocook', '@juggua', '@LvMedBr', '@catars_is', '@BeeGranola', '@lLKH_HH',
             '@shiftua', '@Black_water_division', '@Coronavirus_Info_MFA', '@KP_KMR_IS', '@hellokyiv', '@non_fikshn',
             '@UkraineHack', '@NAZK_gov_ua', '@ukrtgcatalog', '@studcouch', '@cybercalm', '@kyiv_traffic',
             '@KPMG_Ukraine', '@nakipelovo', '@cikavisko', '@vasyl_kit', '@coronaviruscomua', '@nails_ua', '@kma_today',
             '@PresidentZelenskij', '@you_can_know_more', '@guliajpole', '@andriysadovyi', '@beloesuhoeUA', '@KastaUa',
             '@englishwordsua', '@ilioutfit', '@wasmedia_ua', '@xaxaua', '@zhpmch', '@UaSportTV', '@osvitanova',
             '@XTOTAM', '@journaltg', '@naurok', '@Ukrainska_Mova', '@junior_dou_ua', '@buddynaukma', '@Dnepr311',
             '@osoblyva', '@zapas_slvEN', '@yakabooua', '@dnvr_31', '@regionality', '@math_znocloub', '@tripmix',
             '@faini_memes', '@vonyporuch', '@ukrainiantraveler', '@the_ukr_news', '@Medvezhatko1488', '@commonscomua',
             '@ternopil1540', '@po_secretyy', '@AlleeECHR', '@desibuy', '@iptkpiguild', '@clockwork_88', '@nails_manik',
             '@commacomua', '@milabaraeva', '@mintsyfra', '@aliexpresukraine', '@lvivVScoronavirus', '@dou_editors',
             '@apkha', '@avto_kh', '@valizakordon', '@KievG', '@manifestinua', '@FootballUK', '@moviesjournal',
             '@noski_ua', '@skaybu', '@nkplusua', '@nacionalist_ua', '@elmundochannel', '@int_planet', '@Hotels24_ua',
             '@KarantynKyiv', '@villageua', '@slovopro', '@top_photo', '@varianty_lviv', '@dietologua',
             '@oblradadnipro', '@npzbvnkngchtn', '@nauinformal', '@vse_pro_vse_ua', '@SlugaNarodu_Official',
             '@lifeandlies', '@topfilms_ua', '@theytoldme', '@cresume', '@prodesign_in_ua', '@brutfoot', '@fmc_ua',
             '@leport', '@Metalist1925_uffs', '@ukrexcerption', '@autokiev', '@shom_oratoryschool', '@Bukovel_Live',
             '@officialeurosolidarity', '@qoutefilms_ua', '@kpi_hostel8', '@kliati_pereklady', '@kpi_fea',
             '@reklama_lviv_ua', '@dekanat_fpm', '@StudHurtDnipra', '@Chtec_ua', '@esportinua', '@ze_team2019',
             '@Velo_kyiv_rides', '@vnugo', '@kniazevych', '@myalgorhythm', '@klounnnnn',
             '@DonbassYasinovatayanaliniiOgnia', '@bbc_ukr', '@vox_ukraine', '@nov1ny_sv1ty', '@knutkt', '@kpi_library',
             '@cucumberapps', '@NK_support', '@vinnytsiaVScoronavirys', '@Porohobotchenko', '@banderamedia',
             '@smartuseronline', '@NG_promotion', '@budmaster_info', '@natsionalKharkiv', '@andriykovi',
             '@zentropa_ukraine', '@mandruy', '@hypermuseum', '@ZhmerPegas', '@digitalkrasa', '@tekso_mood',
             '@upravlinniamolodi', '@aventour_ua', '@robotavidtata', '@playua', '@dou_vinnytsia', '@rentCV',
             '@ukrnastup', '@chesno_vybory', '@torontotv', '@ukrgram', '@JoinUPHT', '@startupcontent', '@pohrib',
             '@yyluchkiv8blog', '@ukrainetv', '@nlo_tv', '@fastnews_ua', '@zik_ua', '@fairytailss', '@Krosogolik',
             '@allgolobutsky', '@vata_show', '@shopzona', '@yarmakandriy', '@ukrazom_org', '@kpiquotes',
             '@kpi_hostel18', '@coronews', '@open_kmbs', '@omriyana', '@exchanger_ua', '@ingeniusua', '@guli_vyshnya',
             '@takavacoffeebuffet', '@ua_ternopil', '@andrey_burenok', '@kinopodgon1', '@ukrain1an_news',
             '@kpi_hostel20', '@cinema_room', '@frankivskcity', '@LastBP', '@Koronavirus_kharkov', '@tickets_ua',
             '@theKIEF', '@espresotb', '@ZNO_Grant_education', '@slukhachua', '@mm_knu', '@kyivmetroalerts', '@factuu',
             '@goryshche', '@footballhubua', '@infeach', '@usedappleua', '@founder_Co', '@ukraina24tv', '@friendsua',
             '@news19cov', '@siryivovk', '@unitcity', '@mixsport_pro', '@docua', '@HR_Kiev_UA', '@pelekh', '@dropgv',
             '@correctariumukr', '@LetsGoHike', '@podiilviv', '@studyhardIT', '@cp_km', '@paniukraine', '@silposilpo',
             '@Kiev_job', '@volunesians_ukraine', '@work_eu', '@citrus_academy', '@ekipazh_svobody', '@havryliakonair',
             '@agroportalua', '@moz_official_koronavirus', '@datarootlabs', '@kursvalutkiev', '@fainakraina',
             '@legio_historica', '@YZSPzashita', '@uabash', '@kraina_ua', '@ukrpayless', '@comfysale', '@boontivnyk',
             '@politoscope', '@oburay', '@osoblyve', '@events_ua', '@Dumskaya', '@loveandbudget', '@tm_ukraine',
             '@Miroslavhai', '@sxodim_Kiev', '@dylema', '@bf_109e', '@nashformat', '@ukraineneveragain', '@eventDP',
             '@volkstum', '@accemedin', '@zabutiphoto', '@mov_tribunal', '@smart_videos_ua', '@police_ua',
             '@KievEventsToday', '@judicial_practice', '@dou_kyiv', '@cho_english', '@typical_journalists',
             '@DynamoKyiv', '@ua_bizidea', '@detector_media', '@milinua', '@technpage', '@fin_day', '@bodybuildingfor',
             '@fin_support', '@top_cars_ua', '@history_ukrfacts', '@pinteresttrends', '@startsomethingnow',
             '@animemes_ua', '@politarenainfo', '@cs_100', '@kudakyiv', '@FemidaLaw', '@tsnua_blogs', '@nulp_memes',
             '@lfyoulviv', '@radiotrek', '@osvitatopnews', '@favbetofficial', '@redactionodua', '@SaleeUA',
             '@kato_garbage', '@appleforcedrop', '@dilovodstvomrpl', '@myimoz', '@LowCostTravel_1001idea',
             '@fishingsecret', '@Stud2work', '@supremecourtua', '@topuanews', '@rektorguberski',
             '@Promocodi_akcii_lviv', '@vovkst', '@estetuka', '@FiFishki', '@pashabroskii', '@lift_ua', '@recordsua',
             '@lvivmodelka', '@prohow_digital', '@samovdoskonalennia', '@horeca_lvivska', '@donorua', '@ukrainer_net',
             '@kneulive', '@lvivcitystandup', '@ua_ultras', '@englishkiev', '@nynipro', '@hot_eng', '@Helpremont_in_ua',
             '@svato_ua', '@Gramotaua', '@dekanat_fiot', '@futuretodayua', '@kinoukrr', '@rishuchireformy',
             '@youcontrolnews', '@minfin_realty', '@reklamnutuy', '@cld_nlu', '@nivki_park', '@neymovirno',
             '@devo4ki_takie_devo4ki', '@mistoxa', '@global_events', '@pocketbookua', '@vseosvita', '@uavisualinfo',
             '@svoboda_radio', '@nubipnules', '@Oleksiy_Honcharuk', '@ukrlib', '@nachasi_com', '@tapit', '@kinoriumUA',
             '@COVID19_Ukraine', '@channel_ua', '@who_europe', '@milanabet1', '@slidstvo_info', '@chasopys',
             '@svitolina', '@StudNaUKMA', '@mosiychuk72', '@programist_ua', '@poyasnyzaartua', '@odessa_kurs',
             '@prikollikes', '@tvoya_psychology_UA', '@V_Zelenskiy_official', '@tvoysohranenki', '@YoungUncorrupt',
             '@hungrykyiv', '@mindua', '@dtkt_ua', '@ist_znocloud', '@solovina', '@jobknu', '@KEF_by',
             '@green_way_com_ua', '@freebooks_ua', '@mebleva_kimnata', '@kpi_xtf', '@scout_dancers_and_models',
             '@zakopejku', '@Motivaciy', '@bizcensor', '@FinClub_ua', '@mentoltea', '@nigtiUA', '@seelifecv',
             '@ukraine_novosti', '@automobilusa', '@nlu_info', '@skido4nik_travel', '@UITua', '@shtravelagency',
             '@thegeeklounge', '@oldyorktimes', '@ukraine_to_nato', '@innowade_marketing', '@krot_1',
             '@yobnunytyy_english', '@parfumsua', '@nowasteukraine', '@karpatylviv', '@dsgnbr', '@notes_christian',
             '@pelenio', '@youmammal', '@smachnoukr', '@design_18', '@spu_knu', '@yurgazeta', '@nabudaily',
             '@polcringe', '@knuevents', '@znua_live', '@yourexua', '@TseHUB', '@ukrazom', '@lifetotravel',
             '@ZKraiuVKrai', '@ntudp', '@vgorode_odessa', '@ukrinform_news', '@MediaLab_Online', '@yuzhva', '@vezuvii',
             '@aschotamupp', '@slovaua', '@freikorps_org', '@business_ukraine', '@ukrfootball_tg', '@zdaiblood',
             '@lowcostavia_travel', '@taxincognito', '@kinobazza', '@diia_gov', '@dzerkalomedia', '@akkermanvasya',
             '@fruty_jigit', '@UCUadmission', '@afisha_u_Lvovi', '@podiilist', '@ukrmems', '@cheaptr_p', '@Ironvoter',
             '@zno_natali', '@marketing_without', '@intromen', '@robotaBC', '@mojamova', '@ucoffee_family',
             '@lifecellukraine', '@fly_UIA', '@mayak_odessa', '@agronews', '@vezhakpi', '@izotope', '@whataflight',
             '@detaliweare', '@pvp_0532', '@rozumschoolua', '@podorozhuy', '@obozrevatelsport', '@lviving',
             '@okwine_ua', '@romabroma', '@dneprnews', '@IASA_Student_Council', '@textbook_ua', '@workuamedia',
             '@UAmonogram', '@Concertua', '@UAtaxesYou', '@DavayLowcostInfo', '@veresclub', '@OpendatabotChannel',
             '@businesseventskyiv', '@jobslviv', '@studydive', '@wlb_gl', '@nerdgasm', '@warmtours', '@aviamarket',
             '@september', '@imthebatya', '@pogana', '@ukr_businessman', '@suspilnerivne', '@MUKGroup', '@navsi200',
             '@Kyiv_Lifestyle', '@ibelieve_ua', '@YeRobotaIF', '@lowcostavia', '@vichirniykyiv', '@poradnyk',
             '@nmu_dopomoga', '@bestkyiv', '@rooms_kyiv', '@nabolelo_odessa', '@bezviztravel', '@Z_Aliexpress',
             '@kyivmaps', '@numstation', '@newssumy', '@npuFSIO', '@samsungukr', '@ryanairnews', '@pingvinpro_news',
             '@ua_rozvynuta', '@Cherkasy_ua', '@mind_live', '@teterivonelove', '@uifuture', '@sharij_odessa',
             '@kyivtrash', '@fabrika_kherson', '@tvoyamova', '@bukovel_resort', '@artylkiev', '@gifsfit',
             '@golda_meir_ukraine', '@project_juventa', '@etiketua', '@m1stud', '@irpininteresting', '@gazetaua_news',
             '@BestSalesKiev', '@tcn_ua', '@podslushano_kpop_ua', '@plomin', '@tuiukraine', '@knuboard',
             '@ukr_mova_online', '@ednist', '@ua_meteo', '@sarcazm_tg', '@kpi_fmm', '@LBuanews',
             '@national_aviation_university', '@thebuchacity', '@tsurv', '@svmotorss', '@lawandevents',
             '@ua_coronavirus', '@localkawo', '@kristinahomenko', '@kryivka', '@ZnoUkr_mova', '@churcher_info',
             '@sobersthoughts', '@morgenxshtern', '@makeyourmind', '@TatoVlog', '@COVlD19_Ukraine', '@agrokebety',
             '@dekanat_mmi', '@ok_spn', '@healthyukraine', '@innovativeplums', '@korrespondentnet', '@rozumets',
             '@dnrpofitsial', '@tginua', '@networkingukraine', '@STUDPOINT', '@OP_UA', '@angelinatroyann', '@lunoteka',
             '@dou_kharkiv', '@mandrivnukposvity', '@schebetyk', '@bazaavtonomerov', '@fourTvComUa', '@uafukraine',
             '@archi_gid_ua', '@chesno_movement', '@primat_kpi', '@Malaahit', '@dekanat_ipsa', '@tiahnybok',
             '@samonerozvytok', '@OriflameUkraine', '@italian_online_in_ua', '@dekanat_fti', '@comfygame', '@SkrypinUA',
             '@adventistyouth', '@ShakhtarD', '@srs_knu', '@bossautoukraine', '@militaryhonor', '@kyivsmartcity',
             '@fashionukr', '@theinsider_ua', '@agrorobota_roslynnytstvo', '@uareview', '@digital_jam',
             '@psyhokorobochka', '@steptv', '@zabava_lviv', '@coronavirus_in_ukraine', '@greenforestschool',
             '@studradaONU', '@ua_fact', '@amoregram', '@PodcastsNOWua', '@bezbroni', '@antikor', '@postironicpol',
             '@ucatua', '@iniazkaraga', '@ukr_pravda', '@vanilla_ua', '@orendalvivn', '@football_ukrain',
             '@FootballPortalNews', '@slukh', '@tokar_ua', '@veter_doit', '@tochkadonbassa', '@filmuagroup',
             '@inbookwetrust', '@odessanovosti', '@postironia_politologia', '@waveua', '@mykharkov_info',
             '@dynamo_news_1927', '@dumyua', '@afishakieva', '@electronics_cs', '@rootsmusic', '@cheapflightsukr',
             '@ulpra', '@curlymovies', '@kpicampus', '@Shmygal_pm', '@martsinkiv_ONLINE', '@ziznannia', '@hroshiua',
             '@EdYurchenko', '@GenuineLeatherOnly', '@marzandpanna', '@easypeasykyiv', '@epamuniprogua', '@kpi_lampa',
             '@ukrbotbot', '@anonchatua', '@dou_lviv', '@breakingnews_ukraine', '@kucherkh', '@konkurentinua',
             '@studparlament_IJ', '@tripmydream_tours_ua', '@Gastroli_News', '@ChernihivOperative', '@StudProfkomONU',
             '@faktylive', '@culinaracademy', '@rabota_ukraina', '@bbcukrainian', '@groshi24', '@cvjobs',
             '@politeyes_rada', '@fisitchan', '@dou_kyiv_events', '@transport_dp', '@TEDinfo', '@lex_info',
             '@biologia_znocloub', '@rivneshtirlits', '@oratorknu', '@dou_dnipro', '@krutoznavstvo', '@BibleQuest',
             '@unicorn_promocodes', '@jobnuta', '@discoutns_UA', '@kriskosyk', '@advocat_ua', '@kyivnochnoy',
             '@rabota_brovary', '@purplesapiens', '@gradusadparnassum', '@ukuderground', '@vyconavec', '@kyivoda',
             '@bavky', '@hdln24', '@interbuh_lz', '@suspilnenews', '@amnesiascope', '@l1shop', '@krasa_Lv',
             '@smolii_ukraine', '@ternopiljob', '@igrosho', '@frazeolohizmy', '@abookua', '@mandrui_ukrainoyu',
             '@sherlockRV', '@pz_zntu', '@MotoristUA', '@hromadskekharkiv', '@icreateinukraine', '@BILL_KNU', '@bitua',
             '@confainua', '@vntu_live', '@apriori_edu', '@SolovinaMova', '@auto_volt', '@L7Trade_Ua', '@ogaeukraine',
             '@PresidentPoroshenko', '@iplankids', '@kitnadroti', '@primetimemagazine', '@UAchannels']



"""
Module work with Keywords
"""
import copy
import string
import re
from datetime import datetime
import trackme.config as config
from trackme.ukr_stemmer import UkrainianStemmer
from trackme.db_connect import mongo


def ukrainian(word):
    """
    Check if all symbols in word are Ukrainian,
    return True if ukrainian
    :param word: string
    :return: Bool
    """
    for letter in word:
        if re.search('[\u0400-\u04FF]', letter):
            continue
        return False
    return True


class Word:
    """
    This class represent only one word
    """

    def __init__(self, word):
        """Initialize word"""
        self.keyword = word['keyword']
        self.links = {}
        self.links['twitter'] = word['links_twitter']
        self.links['telegram'] = word['links_telegram']
        self.links_data = {}
        self.links_data['twitter'] = word['links_twitter_data']
        self.links_data['telegram'] = word['links_telegram_data']
        try:
            self.telegram_info = word['telegram_info']
        except KeyError:
            self.telegram_info = []
        try:
            self.twitter_info = word['twitter_info']
        except KeyError:
            self.twitter_info = []
        self.links_dict = {}
        self.twitter_replies = 0
        self.twitter_likes = 0
        self.twitter_retweets = 0
        self.telegram_views = 0
        self.telegram_reaction = 0
        self.telegram_posts = 0
        self.twitter_posts = 0

    def find_weight(self, text):
        """
        Find weight of a word in text
        :param text: str
        :return: int
        """
        text = text.translate(str.maketrans('', '', string.punctuation))
        text = [UkrainianStemmer(x).stem_word().lower() for x in text.split()]
        weight = text.count(UkrainianStemmer(self.keyword).stem_word().lower())
        return weight

    def check_link(self, text, link, source, info):
        """
        Get weight of the link and add it if needed
        :param text: str
        :param link: str
        :return: None
        """
        weight = self.find_weight(text)
        if weight != 0:
            if source == 'telegram':
                self.telegram_reaction += info[0]
                try:
                    self.telegram_views += int(info[1])
                except ValueError:
                    self.telegram_views += float(info[1][:-1]) * 1000
                self.telegram_posts += 1
            elif source == 'twitter':
                self.twitter_posts += 1
                self.twitter_likes += int(info[0])
                try:
                    self.twitter_retweets += int(info[1])
                except ValueError:
                    self.twitter_retweets += float(info[1][:-1]) * 1000
                self.twitter_replies += int(info[2])

        to_add = True
        for now_link in range(len(self.links[source])):
            now_weight = self.links[source][now_link][0]
            if weight > now_weight:
                self.links[source].insert(now_link, (weight, link, text, info))
                to_add = False
                break
        if len(self.links[source]) < config.NUMBER_WORDS and to_add and weight > 0:
            print("added!")
            self.links[source].append((weight, link, text, info))

        if len(self.links[source]) > config.NUMBER_WORDS:
            self.links[source].pop()

    def transform_to_dict(self):
        """
        Transform links and weight end dict for faster work
        :return: None
        """
        for now in self.links:
            self.links_dict[now[1]] = now

    @staticmethod
    def get_more_data(data):
        """
        Append 0 end the end of data
        if we don't have all information about word
        :param data: list
        :return: list
        """
        data = [int(x) for x in data]
        while len(data) != 7:
            data.append(0)
        return data

    @staticmethod
    def get_telegram_link_list(link):
        """
        Get information about telegram as list
        0 index - link
        1 index - text no more than 300 symbols
        2 index - number of views
        3 index - number of reactions
        :param link: list
        :return: list
        """
        data = [0] * 4
        data[0] = link[1]
        data[1] = link[2][:297]
        if len(link[2]) > 297:
            data[1] += '...'
        data[2] = link[3][0]
        data[3] = link[3][1]
        return data

    @staticmethod
    def get_twitter_link_list(link):
        """
        Get information about twitter as list
        0 index - link
        1 index - text no more than 300 symbols
        2 index - number of likes
        3 index - number of comments
        4 index - number of shares
        :param link: list
        :return: list
        """
        data = [0] * 6
        data[0] = link[1]
        data[1] = link[2][:297]
        if len(link[2]) > 297:
            data[1] += '...'
        data[2] = link[3][0]
        data[3] = link[3][1]
        data[4] = link[3][2]
        return data

    def transform(self, data):
        """
        Make bold keyword
        :param data: list
        :return: list
        """
        word = UkrainianStemmer(self.keyword).stem_word().lower()
        main_text = copy.copy(data[1]).split(' ')
        data[1] = [UkrainianStemmer(x).stem_word().lower() for x in data[1].split(' ')]
        data[1] = [x.translate(str.maketrans('', '', string.punctuation)) for x in data[1]]
        for i in range(len(data[1])):
            if data[1][i] == word:
                main_text[i] = '<b style="font-weight:750">{}</b>'.format(main_text[i])
        data[1] = ' '.join(main_text)
        return data

    def get_info(self):
        """
        Get full information about keyword
        :return: dict
        """
        data = {}
        if self.telegram_info == list():
            return data
        week_tg = self.telegram_info[:7]
        week_tw = self.twitter_info[:7]
        data['telegram_views'] = self.get_more_data([x['telegram_views'] for x in week_tg])
        data['telegram_reaction'] = self.get_more_data([x['telegram_reaction'] for x in week_tg])
        data['telegram_posts'] = self.get_more_data([x['telegram_posts'] for x in week_tg])
        data['twitter_replies'] = self.get_more_data([x['twitter_replies'] for x in week_tw])
        data['twitter_likes'] = self.get_more_data([x['twitter_likes'] for x in week_tw])
        data['twitter_retweets'] = self.get_more_data([x['twitter_retweets'] for x in week_tw])
        data['twitter_posts'] = self.get_more_data([x['twitter_posts'] for x in week_tw])
        data_tg = self.links_data['telegram']
        data_tw = self.links_data['twitter']
        data['telegram_links'] = [self.transform(self.get_telegram_link_list(x)) for x in data_tg]
        data['twitter_links'] = [self.transform(self.get_twitter_link_list(x)) for x in data_tw]
        return data

    def __str__(self):
        """
        String representation of a word.
        :return: str
        """
        return "%s" % (self.keyword)


class Keywords:
    """
    This class represent all Keywords in list
    """

    def __init__(self):
        """
        Get all keywords from db
        """
        self.keywords = {}
        all_keywords = mongo.db.keywords.find({})
        for keyword in all_keywords:
            self.keywords[keyword['keyword']] = Word(keyword)

    def _check(self, word):
        """
        Check if smb already use this word
        :param word: str
        :return: bool
        """
        return word in self.keywords

    def add(self, word):
        """
        Add word end db and class
        :param word: str
        :return: None
        """
        if not mongo.db.keywords.find_one({'keyword': word}):
            mongo.db.keywords.insert({'keyword': word, 'links_twitter': [],
                                      'links_telegram': [], 'telegram_info': [],
                                      'twitter_info': [],
                                      'links_twitter_data': [],
                                      'links_telegram_data': []})
            self.keywords[word] = Word({'keyword': word, 'links_twitter': [],
                                        'links_telegram': [], 'telegram_info': [],
                                        'twitter_info': [],
                                        'telegram_views': 0, 'telegram_reaction': 0,
                                        'telegram_posts': 0,
                                        'twitter_replies': 0, 'twitter_likes': 0,
                                        'twitter_retweets': 0,
                                        'twitter_posts': 0, 'links_twitter_data': [],
                                        'links_telegram_data': []})

    def __getitem__(self, item):
        """
        Get word from keylist
        :param item: str
        :return: word
        """
        return self.keywords[item]

    def add_new_link(self, text, link, source, info):
        """
        Add new link end all words
        :param text: str
        :param link: str
        :return: None
        """
        print("Adding new link")
        for word in self.keywords:
            word = self.keywords[word]
            word.check_link(text, link, source, info)
            word.transform_to_dict()

    def __str__(self):
        """
        String representation of keywords.
        :return: str
        """
        output = ""
        for word in self.keywords:
            output += str(word) + "\n"
        return output

    def push_changes(self, source):
        """
        Push changes end database
        :param source: str
        :return: None
        """
        for word in self.keywords:
            word = self.keywords[word]
            telegram_info = word.telegram_info
            twitter_info = word.twitter_info
            if source == 'telegram':
                telegram_info.insert(0, {
                    'data': str(datetime.today())[:10],
                    'telegram_views': word.telegram_views,
                    'telegram_reaction': word.telegram_reaction,
                    'telegram_posts': word.telegram_posts})
                mongo.db.keywords.update({"keyword": word.keyword}, {"$set": {
                    "links_telegram": word.links['telegram'],
                    'telegram_info': telegram_info}})
            else:
                twitter_info.insert(0, {
                    'data': str(datetime.today())[:10],
                    'twitter_posts': word.twitter_posts,
                    'twitter_replies': word.twitter_replies,
                    'twitter_likes': word.twitter_likes,
                    'twitter_retweets': word.twitter_retweets})

                mongo.db.keywords.update({"keyword": word.keyword}, {"$set": {
                    "links_twitter": word.links['twitter'],
                    'twitter_info': twitter_info}})

    def clean_changes(self, source):
        """
        Clean previous day changes
        :param source: str
        :return: None
        """
        for word in self.keywords:
            word = self.keywords[word]
            if source == 'telegram':
                mongo.db.keywords.update({"keyword": word.keyword}, {"$set": {
                    "links_telegram": [],
                    "links_telegram_data": word.links['telegram'][:5]}})
            else:
                mongo.db.keywords.update({"keyword": word.keyword}, {"$set": {
                    "links_twitter": [],
                    "links_twitter_data": word.links['twitter'][:5]}})
        self.keywords = {}
        all_keywords = mongo.db.keywords.find({})
        for keyword in all_keywords:
            self.keywords[keyword['keyword']] = Word(keyword)



"""
Telegram parsing module
"""
import time
from datetime import datetime, timedelta
import requests
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.common.by import By
from selenium.common.exceptions import TimeoutException
from trackme.twitter_parsing.twitter_parse import send
from trackme.config import logger
from trackme.telegram_parsing.channels import channels

def parse_telegram(parser):
    """
    Main Telegram channels parsing function.
    :param parser: Parser
    :return: None
    """
    start = time.time()
    browser = parser.browser

    to_parse = [get_date(datetime.now() - timedelta(days=1))]
    dates = [get_date(datetime.now())]
    logger.info("Starting parsing process. Dates: %str_check", dates)
    missed = set()
    got_text = 0

    for num, source in enumerate(channels):
        url = 'https://t.me/s/' + source[1:]

        response = requests.get(url)
        if not response.url.startswith('https://t.me/s/'):  # catch redirect
            logger.warning("Channel parsing not working!")
            continue

        browser.get(url)

        try:
            WebDriverWait(browser, 3).until(
                expected_conditions.presence_of_element_located(
                    (By.CLASS_NAME, 'tgme_widget_message')))
        except TimeoutException:
            logger.error("Error while parsing! %s" % url)
            continue
        posts = browser.find_element_by_class_name('tgme_channel_history') \
            .find_elements_by_class_name('tgme_widget_message_wrap')
        posts.reverse()
        for post in posts:
            try:
                post_date = post.find_element_by_class_name('tgme_widget_message'
                                                            '_service_date_wrap') \
                    .find_element_by_class_name('tgme_widget_message_service_date')
            except:
                send("Error with url " + url)
                continue

            post_date = post_date.get_attribute('innerHTML')
            if post_date not in dates and post_date not in to_parse:
                break
            elif post_date in dates:
                continue

            post = post.find_element_by_class_name('tgme_widget_message')
            post_id = post.get_attribute('data-post').split('/')[1]

            link = parser.by_class(post, 'tgme_widget_message_link_preview')
            text = parser.by_class(post, 'tgme_widget_message_text', True)
            button_link = parser. \
                by_class(post, 'tgme_widget_message_inline_button')
            external_link_text = parser. \
                by_class(post, 'tgme_widget_message_inline_button_text', True, True)
            reactions = 0
            if external_link_text:

                for element in external_link_text:
                    try:
                        btn_num = element.text.split()[-1]
                        if btn_num.endswith("K"):
                            btn_num = float(btn_num[:-1]) * 1000
                        btn_num = int(btn_num)
                        reactions += btn_num
                    except ValueError:
                        continue

            views = parser.by_class(post, 'tgme_widget_message_views', True)

            if text:
                got_text += 1
            if link:
                link = link.get_attribute('href')
            if button_link:
                button_link = button_link.get_attribute('href')

            if text:
                parser.new_link(text, 'https://t.me/' + source[1:] + '/' +
                                str(post_id), 'telegram', (reactions, views))

            logger.info("%str_check %str_check %str_check %str_check "
                        "%str_check %str_check %str_check", 'https://t.me/' + source[1:] + '/' +
                        str(post_id), views, reactions, link,
                        button_link, external_link_text, text)
            print("reactions", reactions)

        if num % 50 == 0:
            send("Parsed %s TG channels out of %s" % (num, len(channels)))

    parse_data = "Parsing process finished. Result:\n%s missed channels;\n" \
                 "%s new posts retrieved;\nTotal of %s channels parsed;\n" \
                 "Total time taken: %.2f minutes" % \
                 (len(missed), got_text, len(channels) - len(missed),
                  (time.time() - start) / 60)

    logger.info(parse_data)
    send(parse_data)





def get_date(date):
    """
    Getting date in the format %B %d.
    :param date: datetime.datetime
    :return: str
    """
    return date.strftime("%B %d").replace(" 0", " ")



# coding=utf8
"""
Main module run it end run application
"""
import bcrypt
from flask import Flask, render_template, url_for, request, session, redirect, flash
from trackme.forms import LoginForm, RegistrationForm
import trackme.config as config
from trackme.db_connect import mongo
from trackme.classes.user import to_class, User
from trackme.classes.keyword import ukrainian

app = Flask(__name__)
app.secret_key = config.flask_key
app.config['SECRET_KEY'] = config.secret_key
app.config['MONGO_DBNAME'] = config.mongoname


@app.route('/')
def index():
    """
    Home page
    :return: html_documentation
    """
    if 'user' in session:
        try:
            user = User(session['user'])
        except NameError:
            return redirect(url_for('login'))
        data = {'username': session['user'], 'keywords': user.get_full_data(),
                'telegram_links': user.get_pretty_links('telegram'),
                'twitter_links': user.get_pretty_links('twitter')}
        return render_template('index.html', username=session['user'], data=data)
    return redirect(url_for('login'))


@app.route('/register', methods=['POST', 'GET'])
def register():
    """
    Register page
    :return: html_documentation
    """
    form = RegistrationForm()
    print()
    if form.validate_on_submit():
        users = mongo.db.users
        hashpass = bcrypt.hashpw(form.password.data.encode('utf-8'), bcrypt.gensalt())

        users.insert({'name': form.username.data,
                      'email': form.email.data, 'password': hashpass,
                      'keywords': [], 'links_twitter': [],
                      'links_telegram': []})
        session['user'] = form.username.data
        print(session['user'])
        return redirect(url_for('index'))
    if request.method == 'POST':
        flash("Невдалось стоворити користувача,\n перевірте чи усі дані введено коректно", 'danger')

    return render_template('main_register.html', title='Register', form=form)


@app.route('/login', methods=['POST', 'GET'])
def login():
    """
    Login Page
    :return: html_documentation
    """
    form = LoginForm()
    if form.validate_on_submit():
        users = mongo.db.users
        login_user = users.find_one({'email': form.email.data})

        if login_user:
            if bcrypt.hashpw(form.password.data.encode('utf-8'), login_user['password']) \
                    == login_user['password']:
                session['user'] = login_user['name']
                return redirect(url_for('index'))
        flash("Неправильна пошта або пароль!", 'danger')
    elif not form.validate_on_submit() and request.method == 'POST':
        flash("Неправильна пошта або пароль!", 'danger')
    return render_template('main_login.html', title='Register', form=form)


@app.route('/add', methods=['POST'])
def add():
    """
    Add new keyword for user
    :return: redirect end Home page
    """
    if request.method == 'POST':
        user = to_class(session['user'])
        if request.form['keyword'] in user.keywords:
            flash("Це слово уже є у вашому словнику", 'danger')
        elif len(request.form['keyword'].split()) > 1:
            flash("Введіть рівно одне слово!", 'danger')
        elif not ukrainian(request.form['keyword']):
            flash("Цей сайт працює тільки з українськими словами", 'danger')
        else:
            user.add_keyword(request.form['keyword'])
            flash("Це слово додано у ваш словник", 'success')
    return redirect(url_for('index'))


@app.route('/logout', methods=['POST'])
def logout():
    """
    Logout user
    :return: redirect end Login Page
    """
    if request.method == 'POST':
        del session['user']
    return redirect(url_for('login'))


def run():
    app.run(port=5001)


if __name__ == '__main__':
    run()



"""
Twitter parsing module
"""
import time
from datetime import datetime, timedelta
import requests
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.common.by import By
from selenium.common.exceptions import  StaleElementReferenceException, \
    NoSuchElementException
from trackme.config import BOT_TOKEN, logger


def send(text):
    """
    Send message to telegram
    :param text: str
    :return: None
    """
    requests.get(
        'https://api.telegram.org/bot%s/sendMessage?chat_id=138918380&text=%s'
        % (BOT_TOKEN, text))
    requests.get(
        'https://api.telegram.org/bot%s/sendMessage?chat_id=426055346&text=%s'
        % (BOT_TOKEN, text))


def get_tweet_interactions(tweet, interaction):
    """
    Returns interaction count based on its name and tweet data
    :param tweet: Selenium WebElement
    :param interaction: str (like, retweet, reply)
    :return: str
    """
    count = tweet.find_element_by_css_selector(
        'div[data-testid="' + interaction + '"]')
    count = count.get_attribute('aria-label')
    count = [int(s) for s in count.split() if s.isdigit()]
    if not count:
        count = [0]
    return count[0]


def parse_tweets(browser, all_tweets, parser):
    """
    Parse all tweets from search page.
    :param browser: Selenium browser
    :param all_tweets: set
    :return: bool or str. True if parsing is over. False otherwise.
    'error' if an error message appears
    """
    print("parse_tweets")
    tweets = browser.find_elements_by_css_selector('div[data-testid="tweet"]')
    for tweet in tweets:
        try:
            acc = tweet.find_element_by_css_selector('div[dir="ltr"]').\
                find_element_by_tag_name('span').text
            time_posted = tweet.find_element_by_tag_name('time').text
            links = tweet.find_elements_by_css_selector('a[role="link"]')
            for link in links:
                url = link.get_attribute('href')
                if len(url.split('/')) == 6:
                    print(url)
                    break
            likes = get_tweet_interactions(tweet, "like")
            retweets = get_tweet_interactions(tweet, "retweet")
            replies = get_tweet_interactions(tweet, "reply")
            logger.info("likes_count: %str_check", likes)
            logger.info("retweets_count: %str_check", retweets)
            logger.info("reply_count: %str_check", replies)
        except (StaleElementReferenceException, NoSuchElementException):
            continue
        except Exception as err:
            logger.error("UNEXPECTED: " + str(err))
            return 'error'
        tweet_unique_info = acc, time_posted

        if tweet_unique_info not in all_tweets:
            text = ""
            try:
                spans = tweet.find_element_by_css_selector('div[lang="uk"]'). \
                    find_elements_by_tag_name('span')
                for child in spans:
                    text += child.text
                print(text)
            except (StaleElementReferenceException, NoSuchElementException):
                logger.info("No Ukrainian text found!")
                continue

            if text:
                parser.new_link(text, url, "twitter", (likes, retweets, replies))
                all_tweets.add(tweet_unique_info)

        if "m" not in time_posted or "h" not in time_posted:
            return True
    return False


def parse_twitter(parser):
    """
    Main Twitter posts parsing function.
    :param parser: Parser
    :param keywords: list
    :return: None
    """
    start = time.time()
    print(start)
    keywords = parser.keywords.keywords
    browser = parser.browser
    available_time = 10 * 60 * 60 / len(keywords)  # 30 minutes total available
    logger.info("Maximum available time per word: %s seconds" % available_time)

    yesterday = datetime.now() - timedelta(days=1)
    yesterday = yesterday.strftime("  %d").replace(" 0", "")
    logger.info("Starting parsing process. yesterday: %str_check", yesterday)
    send("Starting parsing process.")
    proxies_iterations = 0

    for ind, keyword in enumerate(keywords):
        print(keyword)
        i = 0
        loaded = False
        while not loaded:
            try:
                url = 'https://twitter.com/search?q=' + keyword + '%20lang%3Auk&f=live'
                browser.get(url)
                WebDriverWait(browser, 7).until(
                    expected_conditions.presence_of_element_located(
                        (By.CSS_SELECTOR,
                         'section[aria-labelledby="accessible-list-0"]')))
                loaded = True
            except:
                logger.error("Error with proxy! Trying %s time" % i)
                i += 1
                proxies_iterations += 1
                if i % 20 == 0:
                    logger.info("Refreshing proxy list")
                    parser.browser = parser.browser_setup(iterator=proxies_iterations,
                                                          update_proxies=True)
                elif i % 2 == 0:
                    parser.browser = parser.browser_setup(proxies_iterations)

        if not loaded:
            continue

        main_start = time.time()
        print(main_start)

        all_tweets = set()
        finished = False
        last_len = 0
        stop_parsing_count = 0
        iterations = 0
        while not finished and time.time() - main_start < available_time * (ind + 1):
            finished = parse_tweets(browser, all_tweets, parser)
            if finished == 'error':
                logger.info("IP Address Error. Changing it...")
                send("IP Address Error. Changing it...")
                proxies_iterations += 1
                parser.browser = parser.browser_setup(proxies_iterations)
                finished = True
                continue
            browser.execute_script("window.scrollTo(0,  document.body.scrollHeight)")
            print(iterations, stop_parsing_count, len(all_tweets),
                  (time.time() - start) / 60, all_tweets)
            if len(all_tweets) == last_len:
                stop_parsing_count += 1
            else:
                stop_parsing_count = 0
            if stop_parsing_count == 100:
                logger.info("IP Address exhausted. Changing it...")
                send("IP Address exhausted. Changing it...")
                send("Parsed %s tweets for word %s" % (last_len, keyword))
                proxies_iterations += 1
                parser.browser = parser.browser_setup(proxies_iterations)
                finished = True
                continue
            iterations += 1
            last_len = len(all_tweets)

        send("Parsed %s tweets for word %s. Taken %s minutes" %
             (len(all_tweets), keyword, (time.time() - main_start)/60))



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()
txt = '''selenium==3.141.0
Flask==1.1.1    
Flask-WTF==0.14.2
pymongo==3.9.0
WTForms==2.2.1 
urllib3==1.24.1
APScheduler==3.6.3
dnspython==1.16.0
http_request_randomizer==1.2.3
'''
required = txt.splitlines()
setuptools.setup(
    name='trackme',
    version='1.0.5',
    author='Bohdan Vey and Dmytro Lopushanskyy',
    description='Social media monitoring',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/DmytroLopushanskyy/Social-Media-Monitoring',
    packages=setuptools.find_packages(exclude=['examples']),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    install_requires=required
)



"""
This code is already uploaded in Github:
https://github.com/Amice13/ukr_stemmer



Russian stemming algorithm provided by Dr Martin Porter (snowball.tartarus.org):
http://snowball.tartarus.org/algorithms/russian/stemmer.html
Algorithm implementation in PHP provided by Dmitry Koterov (dklab.ru):
http://forum.dklab.ru/php/advises/HeuristicWithoutTheDictionaryExtractionOfARootFromRussianWord.html
Algorithm implementation adopted for Drupal by Algenon (4algenon@gmail.com):
https://drupal.org/project/ukstemmer
Algorithm implementation in Python by Zakharov Kyrylo
https://github.com/Amice13
"""

import re


class UkrainianStemmer():
    """
    Class to get origin of the word
    """

    def __init__(self, word):
        """
        Initialize word
        :param word: str
        """
        self.word = word
        self.vowel = r'аеиоуюяіїє'  # http://uk.wikipedia.org/wiki/Голосний_звук
        self.perfectiveground = r'(ив|ивши|ившись|ыв|ывши|ывшись((?<=[ая])(в|вши|вшись)))$'
        # http://uk.wikipedia.org/wiki/Рефлексивне_дієслово
        self.reflexive = r'(с[яьи])$'
        # http://uk.wikipedia.org/wiki/Прикметник + http://wapedia.mobi/uk/Прикметник
        self.adjective = r'(ими|ій|ий|а|е|ова|ове|ів|є|їй|єє|еє|я|ім|ем|им|ім|их|' \
                         r'іх|ою|йми|іми|у|ю|ого|ому|ої|на|ний|но)$'
        # http://uk.wikipedia.org/wiki/Дієприкметник
        self.participle = r'(ий|ого|ому|им|ім|а|ій|у|ою|ій|і|их|йми|их)$'
        # http://uk.wikipedia.org/wiki/Дієслово
        self.verb = r'(сь|ся|ив|ать|ять|у|ю|ав|али|учи|ячи|вши|ши|е|ме|ати|яти|є)$'
        # http://uk.wikipedia.org/wiki/Іменник
        self.noun = r'(а|ев|ов|е|ями|ами|еи|и|ей|ой|ий|й|иям|ям|ием|ем|ам|ом|о|у' \
                    r'|ах|иях|ях|ы|ь|ию|ью|ю|ия|ья|я|і|ові|ї|ею|єю|ою|є|еві|ем|єм|ів|їв|ю)$'
        self.rvre = r'[аеиоуюяіїє]'
        self.derivational = r'[^аеиоуюяіїє][аеиоуюяіїє]+[^аеиоуюяіїє]+[аеиоуюяіїє].*(?<=о)сть?$'
        self.reverse = ''

    @staticmethod
    def ukstemmer_search_preprocess(word):
        """
        Change russian symbols end ukrainian
        :param word:
        :return:
        """
        word = word.lower()
        word = word.replace("'", "")
        word = word.replace("ё", "е")
        word = word.replace("ъ", "ї")
        return word

    def str_check(self, start, reg, end):
        """
        Check string
        :param start: str
        :param reg: int
        :param end: int
        :return:
        """
        orig = start
        self.reverse = re.sub(reg, end, start)
        return orig != self.reverse

    def stem_word(self):
        """
        Get the origin of ukrainian word
        :return: str
        """
        word = self.ukstemmer_search_preprocess(self.word)
        if not re.search('[аеиоуюяіїє]', word):
            stem = word
        else:
            str_p = re.search(self.rvre, word)
            start = word[0:str_p.span()[1]]
            self.reverse = word[str_p.span()[1]:]

            # Step 1
            if not self.str_check(self.reverse, self.perfectiveground, ''):

                self.str_check(self.reverse, self.reflexive, '')
                if self.str_check(self.reverse, self.adjective, ''):
                    self.str_check(self.reverse, self.participle, '')
                else:
                    if not self.str_check(self.reverse, self.verb, ''):
                        self.str_check(self.reverse, self.noun, '')
            # Step 2
            self.str_check(self.reverse, 'и$', '')

            # Step 3
            if re.search(self.derivational, self.reverse):
                self.str_check(self.reverse, 'ость$', '')

            # Step 4
            if self.str_check(self.reverse, 'ь$', ''):
                self.str_check(self.reverse, 'ейше?$', '')
                self.str_check(self.reverse, 'нн$', u'н')

            stem = start + self.reverse
        return stem



"""
 This module used end work with user
"""
import trackme.config as config
from trackme.db_connect import mongo
from trackme.classes.keyword import Keywords


class User:
    """
    Class end work with user easier
    """

    def __init__(self, username):
        """Initialize User"""
        users = mongo.db.users
        login_user = users.find_one({'name': username})
        if login_user is None:
            raise NameError("wrong username")
        self.username = login_user['name']
        self.password = login_user['password']
        self.keywords = login_user['keywords']
        self.email = login_user['email']
        self.weights = {'telegram': [], 'twitter': []}
        self.dict_weights = {}

    def add_keyword(self, new_word):
        """Add keyword end user"""

        self.keywords.append(new_word)
        words = Keywords()
        words.add(new_word)
        mongo.db.users.update({"name": self.username},
                              {"$set": {"keywords": self.keywords}})

    def to_save(self):
        """Return user in json format"""
        return {'name': self.username, 'password': self.password,
                'keywords': self.keywords, 'email': self.email}

    def get_user_weight(self, link):
        """
        Get weight of the link for special user
        :param link: str
        :return: int
        """
        user_weight = 0
        words = Keywords()
        for keyword in self.keywords:
            keyword = words.keywords[keyword]
            if link in keyword.links_dict:
                user_weight += keyword.links_dict[link]
        return user_weight

    def check_user_weight(self):
        """
        Sort weight of the user links end get most popular
        :param link: str
        :return:
        """
        for source in ['telegram', 'twitter']:
            new_links = []
            words = Keywords()
            for now in range(config.NUMBER_WORDS):

                dct = {}
                dct1 = {}
                for link in self.keywords:
                    link = words[link]
                    try:
                        dct[link.links[source][now][1]] = dct.get(link.links[source][now][0], 0) + 1
                        dct1[link.links[source][now][1]] = link.links[source][now]
                    except IndexError:
                        pass
                maxi = 0
                max_link = ''
                for i in dct:
                    try:
                        if dct[i] > maxi and i not in [x[1] for x in new_links]:
                            maxi = dct[i]
                            max_link = dct1[i]
                    except IndexError:
                        pass
                new_links.append(max_link)
            self.weights[source] = new_links

    def update_links(self, source):
        """
        Push changes
        :return:
        """
        self.check_user_weight()
        for ind in range(len(self.weights[source])):
            try:
                self.weights[source][ind][1] = self.weights[source][ind][1].replace('\n', '')
            except IndexError:
                pass
        mongo.db.users.update({"name": self.username},
                              {"$set": {"links_" + source: self.weights[source]}})

    def get_links(self, source):
        """
        Get user links
        :return: list of links
        """
        return mongo.db.users.find_one({"name": self.username})['links_' + source]

    def get_full_data(self):
        """
        Get data that we give end WebPage
        :return: dict
        """
        to_return = {}
        keywords = Keywords()
        for word in self.keywords:
            to_return[word] = keywords[word].get_info()
        return to_return

    def get_pretty_links(self, source):
        """
        Transform link end work with it easier
        :param source: str
        :return: list
        """
        data = self.get_links(source)
        to_return = []
        for ind in data:
            if ind == '':
                continue
            if len(ind[2]) > 300:
                ind[2] = ind[2][:297] + '...'
            to_return.append([ind[1], ind[2], ind[3][0], ind[3][1]])
            if source == 'twitter':
                to_return[-1].append(ind[3][2])
        return to_return



def to_class(session_name):
    """
    Create class from user
    :param session_name: name
    :return: None
    """
    return User(session_name)


def get_all_users():
    """
    Get all users
    :return: list of User
    """
    users = mongo.db.users
    users = users.find({})
    users_name = []
    for user in users:
        users_name.append(User(user['name']))
    return users_name



"""
Config
"""
import logging
secret_key = '5791628bb0b13ce0c676dfde280ba245'
mongoclient = 'mongodb+srv://BohdanVey:newpassword@cluster0-rkq6t.gcp.mongodb' \
              '.net/test?retryWrites=true&w=majority'
mongoname = 'mongologinexample'
NUMBER_WORDS = 20
flask_key = 'newsecretkey'
BOT_TOKEN = "687710974:AAFfjf5i0k4OOrf3m6fauNIq1l4PAB-cTu4"

# GOOGLE_CHROME_BIN = '/app/.apt/usr/bin/google-chrome'
# CHROMEDRIVER_PATH = '/app/.chromedriver/bin/chromedriver'

GOOGLE_CHROME_BIN = '/Applications/Google Chrome.app/Contents/MacOS/Google Chrome'
# CHROMEDRIVER_PATH = '/home/bohdan/Downloads/chromedriver'
CHROMEDRIVER_PATH = '/usr/local/bin/chromedriver'

logger = logging.getLogger("main_logger")